@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -41,14 +41,16 @@
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/headerpath.h>
-#include <qtsupport/debugginghelper.h>
 #include <qtsupport/debugginghelperbuildtask.h>
 #include <qtsupport/qtsupportconstants.h>
 
+#include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/runextensions.h>
 #include <utils/synchronousprocess.h>
+#include <utils/winutils.h>
+#include <utils/algorithm.h>
 
 #include <QDir>
 #include <QUrl>
@@ -137,7 +139,6 @@ int BaseQtVersion::getUniqueId()
 BaseQtVersion::BaseQtVersion(const FileName &qmakeCommand, bool isAutodetected, const QString &autodetectionSource)
     : m_id(getUniqueId()),
       m_isAutodetected(isAutodetected),
-      m_hasDebuggingHelper(false),
       m_hasQmlDump(false),
       m_mkspecUpToDate(false),
       m_mkspecReadUpToDate(false),
@@ -158,7 +159,6 @@ BaseQtVersion::BaseQtVersion(const FileName &qmakeCommand, bool isAutodetected,
 
 BaseQtVersion::BaseQtVersion()
     :  m_id(-1), m_isAutodetected(false),
-    m_hasDebuggingHelper(false),
     m_hasQmlDump(false),
     m_mkspecUpToDate(false),
     m_mkspecReadUpToDate(false),
@@ -233,16 +233,36 @@ FeatureSet BaseQtVersion::availableFeatures() const
             | FeatureSet(Constants::FEATURE_QT_WEBKIT)
             | FeatureSet(Constants::FEATURE_QT_CONSOLE);
 
-     if (qtVersion() >= QtVersionNumber(4, 7, 0)) {
-         features |= FeatureSet(Constants::FEATURE_QT_QUICK);
-         features |= FeatureSet(Constants::FEATURE_QT_QUICK_1);
-     }
-     if (qtVersion() >= QtVersionNumber(4, 7, 1))
-         features |= FeatureSet(Constants::FEATURE_QT_QUICK_1_1);
-     if (qtVersion() >= QtVersionNumber(5, 0, 0))
-         features |= FeatureSet(Constants::FEATURE_QT_QUICK_2);
-     if (qtVersion() >= QtVersionNumber(5, 1, 0))
-         features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS);
+     if (qtVersion() < QtVersionNumber(4, 7, 0))
+         return features;
+
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK);
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_1);
+
+     if (qtVersion() < QtVersionNumber(4, 7, 1))
+         return features;
+
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_1_1);
+
+     if (qtVersion() < QtVersionNumber(5, 0, 0))
+         return features;
+
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2);
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2_0);
+
+     if (qtVersion() < QtVersionNumber(5, 1, 0))
+         return features;
+
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2_1);
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS);
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS_1);
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS_1_0);
+
+     if (qtVersion() < QtVersionNumber(5, 2, 0))
+         return features;
+
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2_2);
+     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS_1_1);
 
      return features;
 }
@@ -297,10 +317,10 @@ QList<Task> BaseQtVersion::validateKit(const Kit *k)
         if (!fullMatch) {
             if (!fuzzyMatch)
                 message = QCoreApplication::translate("BaseQtVersion",
-                                                      "The compiler '%1' (%2) cannot produce code for the Qt version '%3' (%4).");
+                                                      "The compiler \"%1\" (%2) cannot produce code for the Qt version \"%3\" (%4).");
             else
                 message = QCoreApplication::translate("BaseQtVersion",
-                                                      "The compiler '%1' (%2) may not produce code compatible with the Qt version '%3' (%4).");
+                                                      "The compiler \"%1\" (%2) may not produce code compatible with the Qt version \"%3\" (%4).");
             message = message.arg(tc->displayName(), targetAbi.toString(),
                                   version->displayName(), qtAbiString);
             result << Task(fuzzyMatch ? Task::Warning : Task::Error, message, FileName(), -1,
@@ -461,16 +481,14 @@ QStringList BaseQtVersion::warningReason() const
 ToolChain *BaseQtVersion::preferredToolChain(const FileName &ms) const
 {
     const FileName spec = ms.isEmpty() ? mkspec() : ms;
-    ToolChain *possibleTc = 0;
-    foreach (ToolChain *tc, ToolChainManager::toolChains()) {
-        if (!qtAbis().contains(tc->targetAbi()))
-            continue;
-        if (tc->suggestedMkspecList().contains(spec))
-            return tc; // perfect match
-        if (!possibleTc)
-            possibleTc = tc; // first possible match
-    }
-    return possibleTc;
+
+    QList<ToolChain *> toolchains = ToolChainManager::toolChains();
+    return Utils::findOr(toolchains,
+                         toolchains.isEmpty() ? 0 : toolchains.first(),
+                         [&spec, this](ToolChain *tc) {
+                                return qtAbis().contains(tc->targetAbi())
+                                         && tc->suggestedMkspecList().contains(spec);
+                          });
 }
 
 FileName BaseQtVersion::qmakeCommand() const
@@ -697,36 +715,26 @@ QString BaseQtVersion::findQtBinary(Binaries binary) const
 
     QStringList possibleCommands;
     switch (binary) {
-    case QmlScene: {
-        if (HostOsInfo::isWindowsHost())
-            possibleCommands << QLatin1String("qmlscene.exe");
-        else
-            possibleCommands << QLatin1String("qmlscene");
-    }
+    case QmlScene:
+        possibleCommands << HostOsInfo::withExecutableSuffix(QLatin1String("qmlscene"));
+        break;
     case QmlViewer: {
-        if (HostOsInfo::isWindowsHost())
-            possibleCommands << QLatin1String("qmlviewer.exe");
-        else if (HostOsInfo::isMacHost())
+        if (HostOsInfo::isMacHost())
             possibleCommands << QLatin1String("QMLViewer.app/Contents/MacOS/QMLViewer");
         else
-            possibleCommands << QLatin1String("qmlviewer");
+            possibleCommands << HostOsInfo::withExecutableSuffix(QLatin1String("qmlviewer"));
     }
         break;
     case Designer:
-        if (HostOsInfo::isWindowsHost())
-            possibleCommands << QLatin1String("designer.exe");
-        else if (HostOsInfo::isMacHost())
+        if (HostOsInfo::isMacHost())
             possibleCommands << QLatin1String("Designer.app/Contents/MacOS/Designer");
         else
-            possibleCommands << QLatin1String("designer");
+            possibleCommands << HostOsInfo::withExecutableSuffix(QLatin1String("designer"));
         break;
     case Linguist:
-        if (HostOsInfo::isWindowsHost())
-            possibleCommands << QLatin1String("linguist.exe");
-        else if (HostOsInfo::isMacHost())
+        if (HostOsInfo::isMacHost())
             possibleCommands << QLatin1String("Linguist.app/Contents/MacOS/Linguist");
-        else
-            possibleCommands << QLatin1String("linguist");
+            possibleCommands << HostOsInfo::withExecutableSuffix(QLatin1String("linguist"));
         break;
     case Uic:
         if (HostOsInfo::isWindowsHost()) {
@@ -911,7 +919,6 @@ void BaseQtVersion::updateVersionInfo() const
     m_installed = true;
     m_hasExamples = false;
     m_hasDocumentation = false;
-    m_hasDebuggingHelper = false;
     m_hasQmlDump = false;
 
     if (!queryQMakeVariables(qmakeCommand(), qmakeRunEnvironment(), &m_versionInfo)) {
@@ -927,7 +934,6 @@ void BaseQtVersion::updateVersionInfo() const
     const QString qtHeaderData = qmakeProperty(m_versionInfo, "QT_INSTALL_HEADERS");
     if (!qtInstallData.isNull()) {
         if (!qtInstallData.isEmpty()) {
-            m_hasDebuggingHelper = !DebuggingHelperLibrary::debuggingHelperLibraryByInstallData(qtInstallData).isEmpty();
             m_hasQmlDump
                     = !QmlDumpTool::toolForQtPaths(qtInstallData, qtInstallBins, qtHeaderData, false).isEmpty()
                     || !QmlDumpTool::toolForQtPaths(qtInstallData, qtInstallBins, qtHeaderData, true).isEmpty();
@@ -1072,13 +1078,6 @@ Environment BaseQtVersion::qmakeRunEnvironment() const
     return Environment::systemEnvironment();
 }
 
-bool BaseQtVersion::hasGdbDebuggingHelper() const
-{
-    updateVersionInfo();
-    return m_hasDebuggingHelper;
-}
-
-
 bool BaseQtVersion::hasQmlDump() const
 {
     updateVersionInfo();
@@ -1114,14 +1113,6 @@ Environment BaseQtVersion::qmlToolsEnvironment() const
     return environment;
 }
 
-QString BaseQtVersion::gdbDebuggingHelperLibrary() const
-{
-    QString qtInstallData = qmakeProperty("QT_INSTALL_DATA");
-    if (qtInstallData.isEmpty())
-        return QString();
-    return DebuggingHelperLibrary::debuggingHelperLibraryByInstallData(qtInstallData);
-}
-
 QString BaseQtVersion::qmlDumpTool(bool debugVersion) const
 {
     const QString qtInstallData = qmakeProperty("QT_INSTALL_DATA");
@@ -1132,21 +1123,6 @@ QString BaseQtVersion::qmlDumpTool(bool debugVersion) const
     return QmlDumpTool::toolForQtPaths(qtInstallData, qtInstallBins, qtHeaderData, debugVersion);
 }
 
-QStringList BaseQtVersion::debuggingHelperLibraryLocations() const
-{
-    QString qtInstallData = qmakeProperty("QT_INSTALL_DATA");
-    if (qtInstallData.isEmpty())
-        return QStringList();
-    return DebuggingHelperLibrary::debuggingHelperLibraryDirectories(qtInstallData);
-}
-
-bool BaseQtVersion::supportsBinaryDebuggingHelper() const
-{
-    if (!isValid())
-        return false;
-    return true;
-}
-
 void BaseQtVersion::recheckDumper()
 {
     m_versionInfoUpToDate = false;
@@ -1186,7 +1162,7 @@ QList<Task> BaseQtVersion::reportIssuesImpl(const QString &proFile, const QStrin
     const QChar slash = QLatin1Char('/');
     if (!sourcePath.endsWith(slash))
         sourcePath.append(slash);
-    if ((tmpBuildDir.startsWith(sourcePath)) && (tmpBuildDir != sourcePath)) {
+    if ((tmpBuildDir.startsWith(sourcePath)) && (tmpBuildDir != sourcePath) && qtVersion() < QtVersionNumber(5, 2, 0)) {
         const QString msg = QCoreApplication::translate("QmakeProjectManager::QtVersion",
                                                         "Qmake does not support build directories below the source directory.");
         results.append(Task(Task::Warning, msg, FileName(), -1,
@@ -1205,7 +1181,7 @@ QList<Task> BaseQtVersion::reportIssuesImpl(const QString &proFile, const QStrin
 QList<Task> BaseQtVersion::reportIssues(const QString &proFile, const QString &buildDir) const
 {
     QList<Task> results = reportIssuesImpl(proFile, buildDir);
-    qSort(results);
+    Utils::sort(results);
     return results;
 }
 
@@ -1221,21 +1197,24 @@ static QByteArray runQmakeQuery(const FileName &binary, const Environment &env,
 
     const int timeOutMS = 30000; // Might be slow on some machines.
 
+    // Prevent e.g. qmake 4.x on MinGW to show annoying errors about missing dll's.
+    WindowsCrashDialogBlocker crashDialogBlocker;
+
     QProcess process;
     process.setEnvironment(env.toStringList());
     process.start(binary.toString(), QStringList(QLatin1String("-query")), QIODevice::ReadOnly);
 
     if (!process.waitForStarted()) {
-        *error = QCoreApplication::translate("QtVersion", "Cannot start '%1': %2").arg(binary.toUserOutput()).arg(process.errorString());
+        *error = QCoreApplication::translate("QtVersion", "Cannot start \"%1\": %2").arg(binary.toUserOutput()).arg(process.errorString());
         return QByteArray();
     }
     if (!process.waitForFinished(timeOutMS)) {
         SynchronousProcess::stopProcess(process);
-        *error = QCoreApplication::translate("QtVersion", "Timeout running '%1' (%2 ms).").arg(binary.toUserOutput()).arg(timeOutMS);
+        *error = QCoreApplication::translate("QtVersion", "Timeout running \"%1\" (%2 ms).").arg(binary.toUserOutput()).arg(timeOutMS);
         return QByteArray();
     }
     if (process.exitStatus() != QProcess::NormalExit) {
-        *error = QCoreApplication::translate("QtVersion", "'%1' crashed.").arg(binary.toUserOutput());
+        *error = QCoreApplication::translate("QtVersion", "\"%1\" crashed.").arg(binary.toUserOutput());
         return QByteArray();
     }
 
@@ -1252,7 +1231,7 @@ bool BaseQtVersion::queryQMakeVariables(const FileName &binary, const Environmen
 
     const QFileInfo qmake = binary.toFileInfo();
     if (!qmake.exists() || !qmake.isExecutable() || qmake.isDir()) {
-        *error = QCoreApplication::translate("QtVersion", "qmake '%1' is not an executable.").arg(binary.toUserOutput());
+        *error = QCoreApplication::translate("QtVersion", "qmake \"%1\" is not an executable.").arg(binary.toUserOutput());
         return false;
     }
 
@@ -1463,7 +1442,7 @@ void BaseQtVersion::buildDebuggingHelper(ToolChain *tc, int tools)
     buildTask->showOutputOnError(true);
 
     QFuture<void> task = QtConcurrent::run(&QtSupport::DebuggingHelperBuildTask::run, buildTask);
-    const QString taskName = QCoreApplication::translate("BaseQtVersion", "Building helpers");
+    const QString taskName = QCoreApplication::translate("BaseQtVersion", "Building Debugging Helpers");
     ProgressManager::addTask(task, taskName, "Qt::BuildHelpers");
 }
 
